@@ -9,7 +9,7 @@ GLMakie.activate!()
 
 sizes = [2,3,4,3,2]        # Number of qubits in each register
 T2 = 10.0                  # T2 dephasing time of all qubits
-#TODO F = 0.9                    # Fidelity of the raw Bell pairs
+#TODO F = 0.9              # Fidelity of the raw Bell pairs
 
 sim, network = simulation_setup(sizes, T2)
 
@@ -35,7 +35,7 @@ display(fig)
 
 # record the simulation progress
 step_ts = range(0, 30, step=0.1)
-record(fig, "firstgenrepeater-03.swapper.mp4", step_ts, framerate=10, visible=true) do t
+record(fig, "firstgenrepeater_v2-02.swapper.mp4", step_ts, framerate=10, visible=true) do t
     run(sim, t)
     notify(obs)
     ax.title = "t=$(t)"
