@@ -8,7 +8,11 @@ struct T2Dephasing <: AbstractBackground
     t2
 end
 
-"""A depolarization background."""
+"""A depolarization background.
+
+The `τ` parameter specifies the average time between depolarization events (assuming a Poisson point process).
+I.e. after time `t` the probability for an depolarization event is `1-exp(-t/τ)`.
+"""
 struct Depolarization <: AbstractBackground
     τ
 end
@@ -24,3 +28,8 @@ end
 struct AmplitudeDamping <: AbstractBackground
     τ
 end
+
+# TODO
+# T1T2Noise
+# T1TwirledDecay
+# T1T2TwirledNoise
