@@ -1,5 +1,4 @@
 @testitem "Plotting Cairo" tags=[:plotting_cairo] begin
-include("setup_plotting.jl")
 using CairoMakie
 CairoMakie.activate!()
 
@@ -9,4 +8,7 @@ end
 @testset "arguments and observables and tags" begin
     include("test_plotting_2_tags_observables.jl")
 end
+@testset "background map" begin
+    include("test_plotting_3_maps.jl")
+end
 end
